@@ -31,9 +31,10 @@ static int __init find_str_pc_offset(void)
 }
 
 
-static void __stp_stack_print (struct pt_regs *regs, int verbose, int levels)
+static void __stp_stack_print (struct pt_regs *regs, int verbose, int levels,
+                               struct task_struct *tsk)
 {
-#if defined(CONFIG_FRAME_POINTER)
+#ifdef STP_USE_FRAME_POINTER
 	int		pc_offset = find_str_pc_offset();
 	unsigned long	*fp = (unsigned long *)regs->ARM_fp;
 	unsigned long	*next_fp, *pc;
@@ -68,5 +69,5 @@ static void __stp_stack_print (struct pt_regs *regs, int verbose, int levels)
 
 		fp = next_fp;
 	}
-#endif
+#endif /* STP_USE_FRAME_POINTER */
 }
