@@ -19,6 +19,7 @@ struct stap_task_finder_target { };
 
 #include "syscall.h"
 #include "utrace_compatibility.h"
+#include "task_finder_map.c"
 
 static LIST_HEAD(__stp_task_finder_list);
 
@@ -127,6 +128,19 @@ __stp_utrace_task_finder_target_quiesce(enum utrace_resume_action action,
 					unsigned long event);
 #endif
 
+#ifdef UTRACE_ORIG_VERSION
+static u32
+__stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs);
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_entry(enum utrace_resume_action action,
+					      struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs);
+#endif
+
 #ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engine,
@@ -166,6 +180,8 @@ stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 	memset(&new_tgt->ops, 0, sizeof(new_tgt->ops));
 	new_tgt->ops.report_death = &__stp_utrace_task_finder_target_death;
 	new_tgt->ops.report_quiesce = &__stp_utrace_task_finder_target_quiesce;
+	new_tgt->ops.report_syscall_entry = \
+		&__stp_utrace_task_finder_target_syscall_entry;
 	new_tgt->ops.report_syscall_exit = \
 		&__stp_utrace_task_finder_target_syscall_exit;
 
@@ -394,6 +410,7 @@ __stp_get_mm_path(struct mm_struct *mm, char *buf, int buflen)
 #define __STP_TASK_BASE_EVENTS	(UTRACE_EVENT(DEATH))
 
 #define __STP_TASK_VM_BASE_EVENTS (__STP_TASK_BASE_EVENTS	\
+				   | UTRACE_EVENT(SYSCALL_ENTRY)\
 				   | UTRACE_EVENT(SYSCALL_EXIT))
 
 /*
@@ -563,43 +580,90 @@ __stp_call_mmap_callbacks(struct stap_task_finder_target *tgt,
 	}
 }
 
-static void
-__stp_call_mmap_callbacks_with_vma(struct stap_task_finder_target *tgt,
-				   struct task_struct *tsk,
-				   struct vm_area_struct *vma)
+
+static struct vm_area_struct *
+__stp_find_file_based_vma(struct mm_struct *mm, unsigned long addr)
 {
-	char *mmpath_buf;
-	char *mmpath;
-	int rc;
+	struct vm_area_struct *vma = find_vma(mm, addr);
 
-	// Allocate space for a path
-	mmpath_buf = _stp_kmalloc(PATH_MAX);
-	if (mmpath_buf == NULL) {
-		_stp_error("Unable to allocate space for path");
-		return;
-	}
-
-	// Grab the path associated with this vma.
-#ifdef STAPCONF_DPATH_PATH
-	mmpath = d_path(&(vma->vm_file->f_path), mmpath_buf, PATH_MAX);
-#else
-	mmpath = d_path(vma->vm_file->f_dentry, vma->vm_file->f_vfsmnt,
-			mmpath_buf, PATH_MAX);
-#endif
-	if (mmpath == NULL || IS_ERR(mmpath)) {
-		rc = -PTR_ERR(mmpath);
-		_stp_error("Unable to get path (error %d) for pid %d",
-			   rc, (int)tsk->pid);
-	}
-	else {
-		__stp_call_mmap_callbacks(tgt, tsk, mmpath, vma->vm_start,
-					  vma->vm_end - vma->vm_start,
-					  (vma->vm_pgoff << PAGE_SHIFT),
-					  vma->vm_flags);
-	}
-	_stp_kfree(mmpath_buf);
+	// I'm not positive why the checking for vm_start > addr is
+	// necessary, but it seems to be (sometimes find_vma() returns
+	// a vma that addr doesn't belong to).
+	if (vma && (vma->vm_file == NULL || vma->vm_start > addr))
+		vma = NULL;
+	return vma;
 }
 
+
+static void
+__stp_call_mmap_callbacks_with_addr(struct stap_task_finder_target *tgt,
+				    struct task_struct *tsk,
+				    unsigned long addr)
+{
+	struct mm_struct *mm;
+	struct vm_area_struct *vma;
+	char *mmpath_buf = NULL;
+	char *mmpath = NULL;
+	long err;
+	unsigned long length;
+	unsigned long offset;
+	unsigned long vm_flags;
+
+	mm = get_task_mm(tsk);
+	if (! mm)
+		return;
+
+	down_read(&mm->mmap_sem);
+	vma = __stp_find_file_based_vma(mm, addr);
+	if (vma) {
+		// Cache information we need from the vma
+		addr = vma->vm_start;
+		length = vma->vm_end - vma->vm_start;
+		offset = (vma->vm_pgoff << PAGE_SHIFT);
+		vm_flags = vma->vm_flags;
+
+		// Allocate space for a path
+		mmpath_buf = _stp_kmalloc(PATH_MAX);
+		if (mmpath_buf == NULL) {
+			_stp_error("Unable to allocate space for path");
+		}
+		else {
+			// Grab the path associated with this vma.
+#ifdef STAPCONF_DPATH_PATH
+			mmpath = d_path(&(vma->vm_file->f_path), mmpath_buf,
+					PATH_MAX);
+#else
+			mmpath = d_path(vma->vm_file->f_dentry,
+					vma->vm_file->f_vfsmnt, mmpath_buf,
+					PATH_MAX);
+#endif
+			if (mmpath == NULL || IS_ERR(mmpath)) {
+				long err = ((mmpath == NULL) ? 0
+					    : -PTR_ERR(mmpath));
+				_stp_error("Unable to get path (error %ld) for pid %d",
+					   err, (int)tsk->pid);
+				mmpath = NULL;
+			}
+		}
+	}
+
+	// At this point, we're done with the vma (assuming we found
+	// one).  We can't hold the 'mmap_sem' semaphore while making
+	// callbacks.
+	up_read(&mm->mmap_sem);
+		
+	if (mmpath)
+		__stp_call_mmap_callbacks(tgt, tsk, mmpath, addr,
+					  length, offset, vm_flags);
+
+	// Cleanup.
+	if (mmpath_buf)
+		_stp_kfree(mmpath_buf);
+	mmput(mm);
+	return;
+}
+
+
 static inline void
 __stp_call_munmap_callbacks(struct stap_task_finder_target *tgt,
 			    struct task_struct *tsk, unsigned long addr,
@@ -919,6 +983,135 @@ __stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
 	return UTRACE_DETACH;
 }
 
+static void
+__stp_call_mmap_callbacks_for_task(struct stap_task_finder_target *tgt,
+				   struct task_struct *tsk)
+{
+	struct mm_struct *mm;
+	char *mmpath_buf;
+	char *mmpath;
+	struct vm_area_struct *vma;
+	int file_based_vmas = 0;
+	struct vma_cache_t {
+#ifdef STAPCONF_DPATH_PATH
+		struct path *f_path;
+#else
+		struct dentry *f_dentry;
+		struct vfsmount *f_vfsmnt;
+#endif
+		unsigned long addr;
+		unsigned long length;
+		unsigned long offset;
+		unsigned long vm_flags;
+	};
+	struct vma_cache_t *vma_cache = NULL;
+	struct vma_cache_t *vma_cache_p; 
+
+	/* Call the mmap_callback for every vma associated with
+	 * a file. */
+	mm = get_task_mm(tsk);
+	if (! mm)
+		return;
+
+	// Allocate space for a path
+	mmpath_buf = _stp_kmalloc(PATH_MAX);
+	if (mmpath_buf == NULL) {
+		mmput(mm);
+		_stp_error("Unable to allocate space for path");
+		return;
+	}
+
+	down_read(&mm->mmap_sem);
+
+	// First find the number of file-based vmas.
+	vma = mm->mmap;
+	while (vma) {
+		if (vma->vm_file)
+			file_based_vmas++;
+		vma = vma->vm_next;
+	}
+
+	// Now allocate an array to cache vma information in.
+	if (file_based_vmas > 0)
+		vma_cache = _stp_kmalloc(sizeof(struct vma_cache_t)
+					 * file_based_vmas);
+	if (vma_cache != NULL) {
+		// Loop through the vmas again, and cache needed information.
+		vma = mm->mmap;
+		vma_cache_p = vma_cache;
+		while (vma) {
+			if (vma->vm_file) {
+#ifdef STAPCONF_DPATH_PATH
+			    // Notice we're increasing the reference
+			    // count for 'f_path'.  This way it won't
+			    // get deleted from out under us.
+			    vma_cache_p->f_path = &(vma->vm_file->f_path);
+			    path_get(vma_cache_p->f_path);
+#else
+			    // Notice we're increasing the reference
+			    // count for 'f_dentry' and 'f_vfsmnt'.
+			    // This way they won't get deleted from
+			    // out under us.
+			    vma_cache_p->f_dentry = vma->vm_file->f_dentry;
+			    dget(vma_cache_p->f_dentry);
+			    vma_cache_p->f_vfsmnt = vma->vm_file->f_vfsmnt;
+			    mntget(vma_cache_p->f_vfsmnt);
+#endif
+			    vma_cache_p->addr = vma->vm_start;
+			    vma_cache_p->length = vma->vm_end - vma->vm_start;
+			    vma_cache_p->offset = (vma->vm_pgoff << PAGE_SHIFT);
+			    vma_cache_p->vm_flags = vma->vm_flags;
+			    vma_cache_p++;
+			}
+			vma = vma->vm_next;
+		}
+	}
+
+	// At this point, we're done with the vmas (assuming we found
+	// any).  We can't hold the 'mmap_sem' semaphore while making
+	// callbacks.
+	up_read(&mm->mmap_sem);
+
+	if (vma_cache) {
+		int i;
+
+		// Loop over our cached information and make callbacks
+		// based on it.
+		vma_cache_p = vma_cache;
+		for (i = 0; i < file_based_vmas; i++) {
+#ifdef STAPCONF_DPATH_PATH
+			mmpath = d_path(vma_cache_p->f_path, mmpath_buf,
+					PATH_MAX);
+			path_put(vma_cache_p->f_path);
+#else
+			mmpath = d_path(vma_cache_p->f_dentry,
+					vma_cache_p->f_vfsmnt, mmpath_buf,
+					PATH_MAX);
+			dput(vma_cache_p->f_dentry);
+			mntput(vma_cache_p->f_vfsmnt);
+#endif
+			if (mmpath == NULL || IS_ERR(mmpath)) {
+				long err = ((mmpath == NULL) ? 0
+					    : -PTR_ERR(mmpath));
+				_stp_error("Unable to get path (error %ld) for pid %d",
+					   err, (int)tsk->pid);
+			}
+			else {
+				__stp_call_mmap_callbacks(tgt, tsk, mmpath,
+							  vma_cache_p->addr,
+							  vma_cache_p->length,
+							  vma_cache_p->offset,
+							  vma_cache_p->vm_flags);
+			}
+			vma_cache_p++;
+		}
+		_stp_kfree(vma_cache);
+	}
+
+	mmput(mm);		/* We're done with mm */
+	_stp_kfree(mmpath_buf);
+}
+
 #ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_quiesce(struct utrace_attached_engine *engine,
@@ -979,98 +1172,31 @@ __stp_utrace_task_finder_target_quiesce(enum utrace_resume_action action,
            don't bother inform map callback clients about its memory map,
            since they will simply duplicate each other. */
 	if (tgt->mmap_events == 1 && tsk->tgid == tsk->pid) {
-		struct mm_struct *mm;
-		char *mmpath_buf;
-		char *mmpath;
-		struct vm_area_struct *vma;
-		int rc;
-
-		/* Call the mmap_callback for every vma associated with
-		 * a file. */
-		mm = get_task_mm(tsk);
-		if (! mm)
-			goto utftq_out;
-
-		// Allocate space for a path
-		mmpath_buf = _stp_kmalloc(PATH_MAX);
-		if (mmpath_buf == NULL) {
-			mmput(mm);
-			_stp_error("Unable to allocate space for path");
-			goto utftq_out;
-		}
-
-		down_read(&mm->mmap_sem);
-		vma = mm->mmap;
-		while (vma) {
-			if (vma->vm_file) {
-#ifdef STAPCONF_DPATH_PATH
-				mmpath = d_path(&(vma->vm_file->f_path),
-						mmpath_buf, PATH_MAX);
-#else
-				mmpath = d_path(vma->vm_file->f_dentry,
-						vma->vm_file->f_vfsmnt,
-						mmpath_buf, PATH_MAX);
-#endif
-				if (mmpath) {
-					__stp_call_mmap_callbacks(tgt, tsk,
-								  mmpath,
-								  vma->vm_start,
-								  vma->vm_end - vma->vm_start,
-								  (vma->vm_pgoff
-								   << PAGE_SHIFT),
-								  vma->vm_flags);
-				}
-				else {
-					_stp_dbug(__FUNCTION__, __LINE__,
-						  "no mmpath?\n");
-				}
-			}
-			vma = vma->vm_next;
-		}
-		up_read(&mm->mmap_sem);
-		mmput(mm);		/* We're done with mm */
-		_stp_kfree(mmpath_buf);
+		__stp_call_mmap_callbacks_for_task(tgt, tsk);
 	}
 
-utftq_out:
 	__stp_tf_handler_end();
 	return UTRACE_RESUME;
 }
 
 
-static struct vm_area_struct *
-__stp_find_file_based_vma(struct mm_struct *mm, unsigned long addr)
-{
-	struct vm_area_struct *vma = find_vma(mm, addr);
-
-	// I'm not positive why the checking for vm_start > addr is
-	// necessary, but it seems to be (sometimes find_vma() returns
-	// a vma that addr doesn't belong to).
-	if (vma && (vma->vm_file == NULL || vma->vm_start > addr))
-		vma = NULL;
-	return vma;
-}
-
 #ifdef UTRACE_ORIG_VERSION
 static u32
-__stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engine,
-					     struct task_struct *tsk,
-					     struct pt_regs *regs)
+__stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs)
 #else
 static u32
-__stp_utrace_task_finder_target_syscall_exit(enum utrace_resume_action action,
-					     struct utrace_attached_engine *engine,
-					     struct task_struct *tsk,
-					     struct pt_regs *regs)
+__stp_utrace_task_finder_target_syscall_entry(enum utrace_resume_action action,
+					      struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs)
 #endif
 {
 	struct stap_task_finder_target *tgt = engine->data;
 	long syscall_no;
-	unsigned long rv;
-	unsigned long args[3];
+	unsigned long args[3] = { 0L };
 	int rc;
-	struct mm_struct *mm;
-	struct vm_area_struct *vma;
 
 	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
 		debug_task_finder_detach();
@@ -1100,62 +1226,97 @@ __stp_utrace_task_finder_target_syscall_exit(enum utrace_resume_action action,
 		&& tgt->munmap_events == 0))
 		return UTRACE_RESUME;
 
-	// Get return value
-	rv = syscall_get_return_value(tsk, regs);
+	__stp_tf_handler_start();
+	if (syscall_no == MUNMAP_SYSCALL_NO(tsk)) {
+		// We need 2 arguments
+		syscall_get_arguments(tsk, regs, 0, 2, args);
+	}
+	else if (syscall_no == MMAP_SYSCALL_NO(tsk)
+		 || syscall_no == MMAP2_SYSCALL_NO(tsk)) {
+		// For mmap, we really just need the return value, so
+		// there is no need to save arguments
+	}
+	else {				// mprotect()
+		// We need 3 arguments
+		syscall_get_arguments(tsk, regs, 0, 3, args);
+	}
 
-	// We need the first syscall argument to see what address we
-	// were operating on.
-	syscall_get_arguments(tsk, regs, 0, 1, args);
+	// Remember the syscall information
+	rc = __stp_tf_add_map(tsk, syscall_no, args[0], args[1], args[2]);
+	if (rc != 0)
+		_stp_error("__stp_tf_add_map returned error %d on pid %d",
+			   rc, tsk->pid);
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
+}
+
+#ifdef UTRACE_ORIG_VERSION
+static u32
+__stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engine,
+					     struct task_struct *tsk,
+					     struct pt_regs *regs)
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_exit(enum utrace_resume_action action,
+					     struct utrace_attached_engine *engine,
+					     struct task_struct *tsk,
+					     struct pt_regs *regs)
+#endif
+{
+	struct stap_task_finder_target *tgt = engine->data;
+	unsigned long rv;
+	struct __stp_tf_map_entry *entry;
+
+	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
+		debug_task_finder_detach();
+		return UTRACE_DETACH;
+	}
+
+	if (tgt == NULL)
+		return UTRACE_RESUME;
+
+	// See if we can find saved syscall info.  If we can, it must
+	// be one of the syscalls we are interested in (and we must
+	// have callbacks to call for it).
+	entry = __stp_tf_get_map_entry(tsk);
+	if (entry == NULL)
+		return UTRACE_RESUME;
+
+	// Get return value
+	__stp_tf_handler_start();
+	rv = syscall_get_return_value(tsk, regs);
 
 #ifdef DEBUG_TASK_FINDER_VMA
 	_stp_dbug(__FUNCTION__, __LINE__,
 		  "tsk %d found %s(0x%lx), returned 0x%lx\n",
 		  tsk->pid,
-		  ((syscall_no == MMAP_SYSCALL_NO(tsk)) ? "mmap"
-		   : ((syscall_no == MMAP2_SYSCALL_NO(tsk)) ? "mmap2"
-		      : ((syscall_no == MPROTECT_SYSCALL_NO(tsk)) ? "mprotect"
-			 : ((syscall_no == MUNMAP_SYSCALL_NO(tsk)) ? "munmap"
+		  ((entry->syscall_no == MMAP_SYSCALL_NO(tsk)) ? "mmap"
+		   : ((entry->syscall_no == MMAP2_SYSCALL_NO(tsk)) ? "mmap2"
+		      : ((entry->syscall_no == MPROTECT_SYSCALL_NO(tsk))
+			 ? "mprotect"
+			 : ((entry->syscall_no == MUNMAP_SYSCALL_NO(tsk))
+			    ? "munmap"
 			    : "UNKNOWN")))),
-		  args[0], rv);
+		  entry->arg0, rv);
 #endif
-	__stp_tf_handler_start();
 
-	if (syscall_no == MUNMAP_SYSCALL_NO(tsk)) {
-		// We need the 2nd syscall argument for the length.
-		syscall_get_arguments(tsk, regs, 1, 1, &args[1]);
+	if (entry->syscall_no == MUNMAP_SYSCALL_NO(tsk)) {
 		// Call the callbacks
-		__stp_call_munmap_callbacks(tgt, tsk, args[0], args[1]);
+		__stp_call_munmap_callbacks(tgt, tsk, entry->arg0, entry->arg1);
 	}
-	else if (syscall_no == MMAP_SYSCALL_NO(tsk)
-		 || syscall_no == MMAP2_SYSCALL_NO(tsk)) {
-
-		mm = get_task_mm(tsk);
-		if (mm) {
-			down_read(&mm->mmap_sem);
-			vma = __stp_find_file_based_vma(mm, rv);
-
-			// Call the callbacks
-			if (vma) {
-				__stp_call_mmap_callbacks_with_vma(tgt, tsk,
-								   vma);
-			}
-
-			up_read(&mm->mmap_sem);
-			mmput(mm);
-		}
-	}
-	else {
-		// We need the 2nd syscall argument for the length and
-		// the 3rd argument for the protection.
-		syscall_get_arguments(tsk, regs, 1, 2, &args[1]);
-
+	else if (entry->syscall_no == MMAP_SYSCALL_NO(tsk)
+		 || entry->syscall_no == MMAP2_SYSCALL_NO(tsk)) {
 		// Call the callbacks
-		__stp_call_mprotect_callbacks(tgt, tsk, args[0], args[1],
-					      args[2]);
+		__stp_call_mmap_callbacks_with_addr(tgt, tsk, rv);
+	}
+	else {				// mprotect
+		// Call the callbacks
+		__stp_call_mprotect_callbacks(tgt, tsk, entry->arg0,
+					      entry->arg1, entry->arg2);
 	}
 
-syscall_exit_done:
 	__stp_tf_handler_end();
+	__stp_tf_remove_map_entry(entry);
 	return UTRACE_RESUME;
 }
 
@@ -1179,6 +1340,8 @@ stap_start_task_finder(void)
 		return ENOMEM;
 	}
 
+        __stp_tf_map_initialize();
+
 	atomic_set(&__stp_task_finder_state, __STP_TF_RUNNING);
 
 	rcu_read_lock();
